from collections.abc import (
    Callable,
    Hashable,
    ItemsView,
    Iterable,
    Iterator,
    KeysView,
    MutableMapping,
    Sequence,
    ValuesView,
)
from typing import Any, Generic, TypeVar, overload
from typing_extensions import Never, Self, TypeAlias

from _typeshed import SupportsKeysAndGetItem

from ._typing import SupportsHashableAndRichComparison

# Without a key fn, Keys must be hashable and comparable
# With a key fn, Keys must be hashable, and the fn converts them to Comparable
# Values can be anything

_T = TypeVar("_T")
_KT = TypeVar("_KT", bound=Hashable)  # Key type.
_VT = TypeVar("_VT")  # Value type.
_KT_co = TypeVar("_KT_co", covariant=True, bound=Hashable)
_VT_co = TypeVar("_VT_co", covariant=True)
_OrderT = TypeVar(
    "_OrderT", bound=SupportsHashableAndRichComparison, contravariant=True
)

KeyFunc: TypeAlias = Callable[[_KT], _OrderT]

class SortedDict(MutableMapping[_KT, _VT]):
    @overload
    def __new__(
        cls,
    ) -> SortedDict[_OrderT, _VT]: ...
    @overload
    def __new__(
        cls,
        __entries: Iterable[tuple[_OrderT, _VT]] | SupportsKeysAndGetItem[_OrderT, _VT],
        /,
    ) -> SortedDict[_OrderT, _VT]: ...
    @overload
    def __new__(
        cls,
        __entries: Iterable[tuple[str, _VT]] | SupportsKeysAndGetItem[str, _VT],
        /,
        **kwargs: _VT,
    ) -> SortedDict[str, _VT]: ...
    @overload
    def __new__(
        cls,
        /,
        **kwargs: _VT,
    ) -> SortedDict[str, _VT]: ...
    @overload
    def __new__(
        cls,
        __key: KeyFunc[str, _OrderT],
        /,
        **kwargs: _VT,
    ) -> SortedKeyDict[str, _VT, _OrderT]: ...
    # mypy 1.5.1 complains: "Overloaded function signatures 5 and 6 overlap with
    # incompatible return types  [misc]". If no kwargs are passed, they would
    # indeed overlap, but there's no way to specify non-empty kwargs. We need
    # this overload to prevent kwargs adding str keys to non-str __entries.
    #
    # Type-enforcement seems to work correct despite the error we ignore here...
    # Alternatively, we would need to not type calls with both entries and
    # kwargs.
    #
    #  Ideally str would be "str or a superclass of str", but I can't find a way
    # to type that.
    @overload
    def __new__(  # type: ignore[misc]
        cls,
        __key: KeyFunc[str, _OrderT],
        __entries: Iterable[tuple[str, _VT]] | SupportsKeysAndGetItem[str, _VT],
        /,
        **kwargs: _VT,
    ) -> SortedKeyDict[str, _VT, _OrderT]: ...
    @overload
    def __new__(
        cls,
        __key: KeyFunc[_KT, _OrderT],
        __entries: Iterable[tuple[_KT, _VT]] | SupportsKeysAndGetItem[_KT, _VT],
        /,
    ) -> SortedKeyDict[_KT, _VT, _OrderT]: ...
    @property
    def key(self) -> KeyFunc[_KT, Any] | None: ...
    @property
    def iloc(self) -> SortedKeysView[_KT]: ...
    def clear(self) -> None: ...
    def __delitem__(self, key: _KT) -> None: ...
    def __getitem__(self, __key: _KT) -> _VT: ...
    def __iter__(self) -> Iterator[_KT]: ...
    def __len__(self) -> int: ...
    def __reversed__(self) -> Iterator[_KT]: ...
    def __setitem__(self, key: _KT, value: _VT) -> None: ...
    def copy(self) -> Self: ...
    def __copy__(self) -> Self: ...
    @classmethod
    @overload
    def fromkeys(cls, seq: Iterable[_OrderT]) -> SortedDict[_OrderT, None]: ...
    @classmethod
    @overload
    def fromkeys(
        cls, seq: Iterable[_OrderT], value: _VT
    ) -> SortedDict[_OrderT, _VT]: ...
    def keys(self) -> SortedKeysView[_KT]: ...
    def items(self) -> SortedItemsView[_KT, _VT]: ...
    def values(self) -> SortedValuesView[_VT]: ...
    @overload
    def pop(self, key: _KT, /) -> _VT: ...
    @overload
    def pop(self, key: _KT, /, default: _T) -> _VT | _T: ...
    def popitem(self, index: int = ...) -> tuple[_KT, _VT]: ...
    def peekitem(self, index: int = ...) -> tuple[_KT, _VT]: ...
    def setdefault(self, key: _KT, default: _VT = ...) -> _VT: ...
    # Our override of update is strictly incompatible in that we only allow
    # kwargs when _KT is str. This is consistent with dict()'s constructor
    # typing, and allowing kwargs with non-str _KT causes TypeErrors at runtime.
    @overload  # type: ignore[override]
    def update(self) -> None: ...
    @overload
    def update(
        self: SortedDict[str, _VT],
        __map: SupportsKeysAndGetItem[str, _VT],
        **kwargs: _VT,
    ) -> None: ...
    @overload
    def update(self, __map: SupportsKeysAndGetItem[_KT, _VT]) -> None: ...
    @overload
    def update(
        self: SortedDict[str, _VT], __iterable: Iterable[tuple[str, _VT]], **kwargs: _VT
    ) -> None: ...
    @overload
    def update(self, __iterable: Iterable[tuple[_KT, _VT]]) -> None: ...
    @overload
    def update(self: SortedDict[str, _VT], **kwargs: _VT) -> None: ...
    def __reduce__(
        self,
    ) -> tuple[
        type[SortedDict[_KT, _VT]],
        tuple[Callable[[_KT], Any], list[tuple[_KT, _VT]]],
    ]: ...
    def bisect_left(self, value: _KT) -> int: ...
    def bisect_right(self, value: _KT) -> int: ...
    def bisect(self, value: _KT) -> int: ...
    def index(
        self, value: _KT, start: int | None = ..., stop: int | None = ...
    ) -> int: ...
    def islice(
        self,
        start: int | None = ...,
        stop: int | None = ...,
        reverse: bool = ...,
    ) -> Iterator[_KT]: ...
    def irange(
        self,
        minimum: _KT | None = ...,
        maximum: _KT | None = ...,
        inclusive: tuple[bool, bool] = ...,
        reverse: bool = ...,
    ) -> Iterator[_KT]: ...
    # Despite being named as a private method, this is documented in the API.
    def _reset(self, load: int) -> None: ...

class SortedKeyDict(SortedDict[_KT, _VT], Generic[_KT, _VT, _OrderT]):
    # This is a stub-only type, it doesn't exist at runtime, so the constructor
    # returns Never to prevent creating instances directly. This also seems to
    # cause a type error if the fromkeys() factory is used from this class, so
    # that doesn't need to be hobbled like this.
    def __new__(
        cls,
        *,
        this_type_only_exists_as_a_type_annotation_it_cannot_be_created_directly: Never,
    ) -> Never: ...
    @property
    def key(self) -> KeyFunc[_KT, _OrderT]: ...
    def irange_key(
        self,
        min_key: _OrderT | None = ...,
        max_key: _OrderT | None = ...,
        inclusive: tuple[bool, bool] = ...,
        reverse: bool = ...,
    ) -> Iterator[_KT]: ...
    def bisect_key_left(self, key: _OrderT) -> int: ...
    def bisect_key_right(self, key: _OrderT) -> int: ...
    def bisect_key(self, key: _OrderT) -> int: ...

class SortedKeysView(KeysView[_KT_co], Sequence[_KT_co]):
    @overload
    def __getitem__(self, index: int) -> _KT_co: ...
    @overload
    def __getitem__(self, index: slice) -> list[_KT_co]: ...
    def __delitem__(self, index: int | slice) -> None: ...

class SortedItemsView(ItemsView[_KT_co, _VT_co], Sequence[tuple[_KT_co, _VT_co]]):
    def __iter__(self) -> Iterator[tuple[_KT_co, _VT_co]]: ...
    @overload
    def __getitem__(self, index: int) -> tuple[_KT_co, _VT_co]: ...
    @overload
    def __getitem__(self, index: slice) -> list[tuple[_KT_co, _VT_co]]: ...
    def __delitem__(self, index: int | slice) -> None: ...

class SortedValuesView(ValuesView[_VT_co], Sequence[_VT_co]):
    @overload
    def __getitem__(self, index: int) -> _VT_co: ...
    @overload
    def __getitem__(self, index: slice) -> list[_VT_co]: ...
    def __delitem__(self, index: int | slice) -> None: ...



from collections.abc import Callable, Hashable, Iterable, Iterator, MutableSet, Sequence
from collections.abc import Set as AbstractSet
from typing import Any, Generic, TypeVar, overload
from typing_extensions import Never, Self, TypeAlias

from ._typing import SupportsHashableAndRichComparison

__all__ = [
    "KeyFunc",
    "SortedKeySet",
    "SortedSet",
    "SupportsHashableAndRichComparison",
]

_T = TypeVar("_T", bound=Hashable)
_OrderT = TypeVar("_OrderT", bound=SupportsHashableAndRichComparison)
KeyFunc: TypeAlias = Callable[[_T], _OrderT]

class SortedSet(MutableSet[_T], Sequence[_T]):
    @overload
    def __new__(cls, iterable: Iterable[_OrderT]) -> SortedSet[_OrderT]: ...
    @overload
    def __new__(cls, key: KeyFunc[_T, _OrderT]) -> SortedKeySet[_T, _OrderT]: ...
    @overload
    def __new__(cls, iterable: None = ..., key: None = ...) -> SortedSet[_OrderT]: ...
    @overload
    def __new__(cls, iterable: Iterable[_OrderT], key: None) -> SortedSet[_OrderT]: ...
    @overload
    def __new__(
        cls,
        iterable: Iterable[_T] | None,
        key: KeyFunc[_T, _OrderT],
    ) -> SortedKeySet[_T, _OrderT]: ...
    @property
    def key(self) -> KeyFunc[_T, Any] | None: ...
    # We have to constrain value to be _T rather than object/Any
    def __contains__(self, value: _T) -> bool: ...  # type: ignore[override]
    @overload
    def __getitem__(self, index: int) -> _T: ...
    @overload
    def __getitem__(self, index: slice) -> list[_T]: ...
    def __delitem__(self, index: int | slice) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __lt__(self, other: Iterable[_T]) -> bool: ...
    def __gt__(self, other: Iterable[_T]) -> bool: ...
    def __le__(self, other: Iterable[_T]) -> bool: ...
    def __ge__(self, other: Iterable[_T]) -> bool: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __reversed__(self) -> Iterator[_T]: ...
    def add(self, value: _T) -> None: ...
    def clear(self) -> None: ...
    def copy(self: Self) -> Self: ...
    def __copy__(self: Self) -> Self: ...
    def count(self, value: _T) -> int: ...
    def discard(self, value: _T) -> None: ...
    def pop(self, index: int = ...) -> _T: ...
    def remove(self, value: _T) -> None: ...
    # difference / intersection methods only subtract from our elements, so they
    # can accept Any.
    def difference(self, *iterables: Iterable[Any]) -> Self: ...
    def __sub__(self, iterable: Iterable[Any]) -> Self: ...
    def difference_update(self, *iterables: Iterable[Any]) -> Self: ...
    def __isub__(self, iterable: Iterable[Any]) -> Self: ...
    def intersection(self, *iterables: Iterable[Any]) -> Self: ...
    def __and__(self, iterable: Iterable[Any]) -> Self: ...
    def __rand__(self, iterable: Iterable[Any]) -> Self: ...
    def intersection_update(self, *iterables: Iterable[Any]) -> Self: ...
    def __iand__(self, iterable: Iterable[Any]) -> Self: ...
    def symmetric_difference(self, other: Iterable[_T]) -> Self: ...
    # The additive dunder methods are incompatible with the superclass because
    # we restrict operand to be T, rather than Any. (We can't compare
    # arbitrary values for order with T.)
    def __xor__(self, other: Iterable[_T]) -> Self: ...  # type: ignore[override]
    def __rxor__(self, other: Iterable[_T]) -> Self: ...
    def symmetric_difference_update(self, other: Iterable[_T]) -> Self: ...
    def __ixor__(self, other: Iterable[_T]) -> Self: ...  # type: ignore[override]
    def union(self, *iterables: Iterable[_T]) -> Self: ...
    def __or__(self, iterable: Iterable[_T]) -> Self: ...  # type: ignore[override]
    def __ror__(self, iterable: Iterable[_T]) -> Self: ...
    def update(self, *iterables: Iterable[_T]) -> Self: ...
    def __ior__(self, iterable: Iterable[_T]) -> Self: ...  # type: ignore[override]
    def __reduce__(
        self,
    ) -> tuple[type[SortedSet[_T]], AbstractSet[_T], Callable[[_T], Any]]: ...
    def bisect_left(self, value: _T) -> int: ...
    def bisect_right(self, value: _T) -> int: ...
    def bisect(self, value: _T) -> int: ...
    def islice(
        self,
        start: int | None = ...,
        stop: int | None = ...,
        reverse: bool = ...,
    ) -> Iterator[_T]: ...
    def irange(
        self,
        minimum: _T | None = ...,
        maximum: _T | None = ...,
        inclusive: tuple[bool, bool] = ...,
        reverse: bool = ...,
    ) -> Iterator[_T]: ...
    def index(
        self, value: _T, start: int | None = ..., stop: int | None = ...
    ) -> int: ...
    # Despite being named as a private method, this is documented in the API.
    def _reset(self, load: int) -> None: ...

# This type doesn't actually exist at runtime, but class SortedSet dynamically
# adds these methods to its instances if it has a key function.
class SortedKeySet(SortedSet[_T], Generic[_T, _OrderT]):
    # This is a stub-only type, it doesn't exist at runtime, so the
    # constructor returns Never to prevent creating instances directly.
    def __new__(
        cls,
        *,
        this_type_only_exists_as_a_type_annotation_it_cannot_be_created_directly: Never,
    ) -> Never: ...
    @property
    def key(self) -> KeyFunc[_T, _OrderT]: ...
    def irange_key(
        self,
        min_key: _OrderT | None = ...,
        max_key: _OrderT | None = ...,
        inclusive: tuple[bool, bool] = ...,
        reverse: bool = ...,
    ) -> Iterator[_T]: ...
    def bisect_key_left(self, key: _OrderT) -> int: ...
    def bisect_key_right(self, key: _OrderT) -> int: ...
    def bisect_key(self, key: _OrderT) -> int: ...



from collections.abc import Hashable
from typing import Any, Protocol
from typing_extensions import TypeAlias

from _typeshed import SupportsDunderGT, SupportsDunderLT, _T_contra  # noqa: F401

class SupportsHashableAndDunderGT(
    Hashable, SupportsDunderGT[_T_contra], Protocol[_T_contra]
): ...
class SupportsHashableAndDunderLT(
    Hashable, SupportsDunderLT[_T_contra], Protocol[_T_contra]
): ...

SupportsHashableAndRichComparison: TypeAlias = (
    SupportsHashableAndDunderLT[Any] | SupportsHashableAndDunderGT[Any]
)

__all__ = ["SupportsHashableAndRichComparison"]



from collections.abc import Callable, Iterable, Iterator, MutableSequence, Sequence
from typing import Any, Final, Generic, TypeVar, overload
from typing_extensions import Self, TypeAlias

from _typeshed import SupportsRichComparison

__all__ = [
    "KeyFunc",
    "_OrderT",
    "SortedKeyList",
    "SortedList",
    "SupportsRichComparison",
]

_T = TypeVar("_T")
_OrderT = TypeVar("_OrderT", bound=SupportsRichComparison)
KeyFunc: TypeAlias = Callable[[_T], _OrderT]

class SortedList(MutableSequence[_T]):
    DEFAULT_LOAD_FACTOR: Final[int] = ...
    @overload
    def __new__(cls, key: KeyFunc[_T, _OrderT]) -> SortedKeyList[_T, _OrderT]: ...
    @overload
    def __new__(cls, iterable: None = ..., key: None = ...) -> SortedList[_OrderT]: ...
    @overload
    def __new__(
        cls, iterable: Iterable[_OrderT], key: None = ...
    ) -> SortedList[_OrderT]: ...
    @overload
    def __new__(
        cls,
        iterable: Iterable[_T] | None,
        key: KeyFunc[_T, _OrderT],
    ) -> SortedKeyList[_T, _OrderT]: ...
    @property
    def key(self) -> KeyFunc[_T, Any] | None: ...
    def clear(self) -> None: ...
    def add(self, value: _T) -> None: ...
    def update(self, iterable: Iterable[_T]) -> None: ...
    def discard(self, value: _T) -> None: ...
    def remove(self, value: _T) -> None: ...
    # The implementation requires that value is T, but the base class accepts
    # Any, which we can't narrow. As a compromise, we narrow anyway and ignore
    # the error. The result is that __contains__ is type safe when using
    # SortedList directly, but not safe if SortedList is assigned to a more
    # general collection type.
    def __contains__(self, value: _T) -> bool: ...  # type: ignore[override]
    def __delitem__(self, index: int | slice) -> None: ...
    @overload
    def __getitem__(self, index: int) -> _T: ...
    @overload
    def __getitem__(self, index: slice) -> list[_T]: ...
    @overload
    def __setitem__(self, index: int, value: _T) -> None: ...
    @overload
    def __setitem__(self, index: slice, value: Iterable[_T]) -> None: ...
    # def __iter__(self) -> Iterator[T]: ...
    def __reversed__(self) -> Iterator[_T]: ...
    def __len__(self) -> int: ...
    # def reverse(self) -> None: ...
    def islice(
        self,
        start: int | None = ...,
        stop: int | None = ...,
        reverse: bool = ...,
    ) -> Iterator[_T]: ...
    def irange(
        self,
        minimum: _T | None = ...,
        maximum: _T | None = ...,
        inclusive: tuple[bool, bool] = ...,
        reverse: bool = ...,
    ) -> Iterator[_T]: ...
    def bisect_left(self, value: _T) -> int: ...
    def bisect_right(self, value: _T) -> int: ...
    def bisect(self, value: _T) -> int: ...
    def count(self, value: _T) -> int: ...
    def copy(self) -> Self: ...
    def __copy__(self) -> Self: ...
    # def append(self, value: T) -> None: ...
    # def extend(self, values: Iterable[T]) -> None: ...
    def insert(self, index: int, value: _T) -> None: ...
    # def pop(self, index: int = ...) -> T: ...
    def index(
        self, value: _T, start: int | None = ..., stop: int | None = ...
    ) -> int: ...
    def __add__(self, other: Iterable[_T]) -> Self: ...
    def __radd__(self, other: Iterable[_T]) -> Self: ...
    def __iadd__(self, other: Iterable[_T]) -> Self: ...
    def __mul__(self, num: int) -> Self: ...
    def __rmul__(self, num: int) -> Self: ...
    def __imul__(self, num: int) -> Self: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __lt__(self, other: Sequence[_T]) -> bool: ...
    def __gt__(self, other: Sequence[_T]) -> bool: ...
    def __le__(self, other: Sequence[_T]) -> bool: ...
    def __ge__(self, other: Sequence[_T]) -> bool: ...
    # Despite being named as a private method, this is documented in the API.
    def _reset(self, load: int) -> None: ...

class SortedKeyList(SortedList[_T], Generic[_T, _OrderT]):
    # The constructor always returns SortedKeyList instances, and uses the
    # identity function as the default key func.
    @overload
    def __new__(
        cls, iterable: Iterable[_OrderT]
    ) -> SortedKeyList[_OrderT, _OrderT]: ...
    @overload
    def __new__(cls, key: KeyFunc[_T, _OrderT]) -> SortedKeyList[_T, _OrderT]: ...
    @overload
    def __new__(cls, iterable: None = ..., key: None = ...) -> Self: ...
    @overload
    def __new__(
        cls, iterable: Iterable[_OrderT], key: None
    ) -> SortedKeyList[_OrderT, _OrderT]: ...
    @overload
    def __new__(
        cls,
        iterable: Iterable[_T] | None,
        key: KeyFunc[_T, _OrderT],
    ) -> SortedKeyList[_T, _OrderT]: ...
    @property
    def key(self) -> KeyFunc[_T, _OrderT]: ...
    def irange_key(
        self,
        min_key: _OrderT | None = ...,
        max_key: _OrderT | None = ...,
        inclusive: tuple[bool, bool] = ...,
        reverse: bool = ...,
    ) -> Iterator[_T]: ...
    def bisect_key_left(self, key: _OrderT) -> int: ...
    def bisect_key_right(self, key: _OrderT) -> int: ...
    def bisect_key(self, key: _OrderT) -> int: ...

SortedListWithKey = SortedKeyList
